# $Id: spanel.tcl 1005 2007-03-04 14:13:16Z sergei $
# Generic horizontal search panel.

##########################################################################

option add *noMatchesBackground		     pink          widgetDefaul

##########################################################################

namespace eval search {}

# TODO (?) add -defdir option to specify def. search dir
# TODO (?) require searchcmd to return a list:
#      [search_result wrapped_around]
#      and signalize wrap-around condition to the user

#
# Recognized options:
# -searchcommand
# -opencommand
# -closecommand
#
# May be?
# -defdir (what direction does Return search)
#
proc search::spanel {w args} {
    set opencmd ""
    set closecmd ""
    foreach {key val} $args {
	switch -- $key {
	    -searchcommand {
		set searchcmd $val
	    }
	    -opencommand {
		set opencmd $val
	    }
	    -closecommand {
		set closecmd $val
	    }
	}
    }

    if {![info exists searchcmd]} {
	error "missing mandatory option: -searchcommand"
    }

    frame $w

    set sentry [entry $w.sentry \
		    -validate all \
		    -validatecommand [namespace code {validate_entry %W %P}]]
    pack $sentry -padx 1m -side left

    set bg [lindex [$sentry configure -background] 4]

    bind $w <Map> [namespace code [list spanel_open [double% $w] \
						    [double% $opencmd]]]

    set sbox [ButtonBox $w.sbox -spacing 0]
    $sbox add -text [::msgcat::mc "Search up"] \
	      -command [namespace code [list spanel_search $sentry $searchcmd up $bg]]
    $sbox add -text [::msgcat::mc "Search down"] \
	      -command [namespace code [list spanel_search $sentry $searchcmd down $bg]]
    pack $sbox -side left -padx 1m

    set cbox [ButtonBox $w.cbox -spacing 0]
    $cbox add -text [::msgcat::mc "Close"] \
	      -command [namespace code [list spanel_close $w $closecmd]]
    pack $cbox -side right -padx 1m

    bind $sentry <Key-Return> [double% [list $sbox invoke 0]]
    bind $sentry <Shift-Key-Return> [double% [list $sbox invoke 1]]
    bind $sentry <Escape> [double% [list $cbox invoke 0]]
    bind $sentry <Escape> +break ;# prevent forwarding upstream

    set w
}

proc search::spanel_search {sentry searchcmd dir dbg} {
    set found [eval $searchcmd [list [$sentry get] $dir]]

    if {$found} {
	set bg $dbg
    } else {
	set bg [option get $sentry noMatchesBackground ""]
	if {$bg == ""} {
	    set bg $dbg
	}
    }

    $sentry configure -background $bg
}

proc search::spanel_open {w opencmd} {
    if {$opencmd != ""} {
	eval $opencmd [list $w]
    }
    focus $w.sentry
}

proc search::spanel_close {w closecmd} {
    if {$closecmd != ""} {
	eval $closecmd [list $w]
    }
}

